! Module      : ommag_mod
!
! Authors     : Tim Harries (MSSL)
!             : Phil Williams (MSSL)
!
! Description : Convert from count-rates to magnitudes in the instrumental system.
!
! Comments    :
!
! $Id: module.f90,v 1.4 1997/12/18 16:01:10 gvacanti Exp $
!
! History : Created  : March 98 - TJH
!           Modified : 20 May 99 - Spring clean of code.
!           Modified : 22/8/2000 (CB) - added call to CAL_omGetColorTrans
!                      to convert instrumental magnitudes to standard magnitudes.
!           modified : 19/9/2000 (AMK) - corrected call to CAL_omPhotoMag. Parameters 
!                      were set incorrectly.
!    4th. October 2000 - AMK
!    Replace the call getFIlterID in mssllib by the OAL call filterIdToInt
!
!
!********************************************************************************
module ommag_mod

   use param
   use cal
   use dal
   use errorHandling
   use frameTime_mod

   implicit none

   private

   public ommag

contains

!---------------------------------------------------------------------
subroutine ommag
!---------------------------------------------------------------------
   character(len=255) :: set                                         ! OM OSW Source List filename.
   character(len=255) :: wdxSet                                      ! OM ODF Priority Window Data Auxiliary filename.
   character(len=8) :: dataMode                                      ! Input file data mode.
   character(len=9) :: filter                                        ! Keywords.

   type(DataSetT) :: inDataSetH                                      ! Input dataset handle.
   type(DataSetT) :: wdxDataSetH                                     ! Priority Window Data dataset handle.
   type(TableT) :: inTableH                                          ! Input table handle.
   type(ColumnT), dimension(6) :: inColumnH                          ! Input image column handle.

   integer(kind=int32) :: nSources                                   ! Number of sources.
   integer(kind=int32) ccdMode                                       ! OM CCD data mode.
   integer(kind=int32) :: nNegative                                  ! Number of -ve count rates.
   integer(kind=int32) :: i                                          ! Count.
   integer(kind=int32) :: filterIdInt
   integer(kind=int8), dimension(:), pointer :: extended

   real(kind=double) :: corrRate                                     ! Corrected count rate.
   real(kind=double) :: corrRateErrPlus                              ! Corrected rate error + (asymmetric).
   real(kind=double) :: corrRateErrMinus                             ! Corrected rate error - (asymmetric).
   real(kind=double) :: corrMag                                      ! Corrected magnitude.
   real(kind=double) :: corrMagErrPlus                               ! Corrected magnitude error + (asymmetric).
   real(kind=double) :: corrMagErrMinus                              ! Corrected magnitude error - (asymmetric).

   real(kind=single), dimension(:), pointer :: rate                  ! Rates pointer.
   real(kind=single), dimension(:), pointer :: rateErr               ! Rates error pointer.

   real(kind=single), dimension(:), pointer :: backGroundRate        ! Source background rate pointer.
   real(kind=double) :: totalCount

   real(kind=single), dimension(:), pointer :: mag                   ! Magnitude pointer.
   real(kind=single), dimension(:), pointer :: magErr                ! Magnitude error pointer.

   real(kind=double) :: frameTime                                    ! OM CCD frame time.
   real(kind=double) :: deadFrac                                     ! Dead time in units of CCD frame time.
   
   real(kind=double)              :: detectionLimitCountRate         ! The detection limit count rate (counts/second)
   real(kind=double)              :: detectionLimitMagnitude         ! The detection limit magnitude
   real(kind=single) :: error = 0.0  
   logical :: extend
 
   !************************
   ! Read in parameters.
   !************************
   set = stringParameter("set")

   wdxSet = stringParameter("wdxset")

   ! Read in the OM OSW Source List file.
   inDataSetH = dataSet(set, modify)
   
         
   if (.not. hasBlock(inDataSetH, "SRCLIST")) then
      !************************************
      ! Incompatible file - can't continue
      !************************************
      write (errstr, *) "Could not find any compatibile blocks in set ", set
      call fatal("badFileType")
   end if

   !**************************************************************
   ! Read in the detection limit count rate from the PRIMARY header
   !**************************************************************
   detectionLimitCountRate = real(real32Attribute(inDataSetH, "CRLIMIT"))
           
   ! Get a handle on the source list binary table extension.
   inTableH = table(inDataSetH, "SRCLIST")

   ! Read in the source list table extension keywords.
   dataMode = stringAttribute(inDataSetH, "DATAMODE")
   filter = stringAttribute(inDataSetH, "FILTER")

   
   !********************************************************
   ! Read in the OM ODF Priority Window Data Auxiliary file.
   !********************************************************
   wdxDataSetH = dataSet(wdxSet, read)

   !************************************************************************
   ! Get the frame time from the OM ODF Priority Window Data Auxiliary file.
   !************************************************************************
   call getFrameTime(wdxDataSetH, frameTime, deadfrac)  ! Frame time is in milli-seconds
   
   frameTime = frameTime / 1000.0    ! Convert frame time to seconds

   !**************************
   ! Release handles on data.
   !**************************
   call release(wdxDataSetH)

   !********************
   ! Get the ccd mode.
   !********************
   if (trim(dataMode) .eq. "IMAGING") then
      ccdMode = IMAGE
   else
      ccdMode = FAST
   end if

   !**************************************
   ! Get the filter Id integer value
   !**************************************
   filterIdInt = filterIdToInt(filter)

   !*******************
   ! Set the CAL state
   !********************
   call CAL_setState(OM, ccdModeId=ccdMode, filterId = filterIdInt)

   !******************************************************
   ! Calculate the instrumental limiting magnitude from
   ! the detection limit count rate
   !******************************************************
   if (detectionLimitCountRate .gt. 0.0) then      

       corrRate = CorrectedCountRate(detectionLimitCountRate, error, frameTime, deadFrac, .false.)
       

       if (corrRate .gt. 0.0) then
          call CAL_omPhotoMagnitude(corrRate, detectionLimitMagnitude, 0.1d0 * corrRate, &
                                   0.1d0 * corrRate, corrMagErrMinus, corrMagErrPlus)
       else
          detectionLimitMagnitude = 0.0d0
       end if
   else
      detectionLimitmagnitude = 0.0
   end if

   call setAttribute(inDataSetH, "MAGLIMIT", real(detectionLimitMagnitude), "", "detection limiting magnitude")


   !******************************************************************
   ! Get the number of rows in the source list binary table extension.
   !******************************************************************
   nSources = numberOfRows(inTableH)
   if (nSources .gt. 0) then


       ! Get handles and pointers on the source list binary table ectension colums.
       inColumnH(1) = column(inTableH, "RATE", modify)
       rate => real32Data(inColumnH(1))

       inColumnH(2) = column(inTableH, "RATE_ERR", modify)
       rateErr => real32Data(inColumnH(2))

       inColumnH(3) = column(inTableH, "MAG", modify)
       mag => real32Data(inColumnH(3))

       inColumnH(4) = column(inTableH, "MAGERR", modify)
       magErr => real32Data(inColumnH(4))

       inColumnH(5) = column(inTableH, "EFLAG", modify)
       extended => int8Data(inColumnH(5))
     
       inColumnH(6) = column(inTableH, "BACKGROUND_RATE", modify)
       backGroundRate => real32Data(inColumnH(6))
   

       !******************************************************************
       ! Loop through all the sources- 
       ! Correct each count-rate for the effect of detector 
       ! lifetime and ageing, then calculate the magnitude.
       !******************************************************************
   
       nNegative = 0   
 
       do i = 0, nSources - 1

           if (extended(i) .eq. 0.0) then
               extend = .false.
           else
               extend = .true.
           end if

           !******************************************************
           ! Need to correct the rates for coincident losses, etc-
           !******************************************************
           totalCount = dble(rate(i) + backGroundRate(i))
   
           corrRate = CorrectedCountRate(totalCount, rateErr(i), frameTime, deadFrac, extend) - &
                  CorrectedCountRate(dble(backGroundRate(i)), error, frameTime, deadFrac, extend) 
  


          if (corrRate .gt. 0.0) then
               call CAL_omPhotoMagnitude(corrRate, corrMag, corrRateErrMinus, &
                                   corrRateErrPlus, corrMagErrMinus, corrMagErrPlus)

               mag(i) = real(corrMag)
               magErr(i) = real(max(corrMagErrMinus, corrMagErrPlus))

           else
               mag(i) = 1.e30
               magErr(i) = 1.e30
               nNegative = nNegative + 1
           end if
       end do
   
       if (nNegative .gt. 0) then
           write (errstr, *) 'There were ', nNegative, ' instances of  negative count-rate'
           call warning("negCounts")
       end if


       !*************************
       ! Release handles on data
       !*************************
       call release(inColumnH(1))
       call release(inColumnH(2))
       call release(inColumnH(3))
       call release(inColumnH(4))
       call release(inColumnH(5))
       call release(inColumnH(6))
   end if                         ! No of sources > 0 block
   call release(inTableH)
   call release(inDataSetH)

end subroutine ommag
!*********************************************************
! Function CorrectedCountRate
! Corrects the raw count rate for coincidence losses, etc  
!*********************************************************
real(kind=double) function CorrectedCountRate(rawCountRate, & 
                  countRateError, frameTime, deadFrac, extended)
    real(kind=double), intent(in)    :: rawCountRate
    real(kind=single), intent(inout) :: countRateError

    real(kind=double), intent(in) :: frameTime
    real(kind=double), intent(in) :: deadFrac
    logical, intent(in) :: extended
    type(ValueAsymmetricErrorT) :: rawRate
    type(ValueAsymmetricErrorT) :: correctedRate

    rawRate%value = rawCountRate
    rawRate%errorMinus = dble(countRateError)
    rawRate%errorPlus = dble(countRateError)

    correctedRate = CAL_omPhotoNatural(rawRate, frameTime, deadFrac, 0.0d0, extended)

    !****************************************************************
    ! If the count rate is greater than the frame rate then the above
    ! routine cannot do the correction- instead it sets the corrected 
    ! count rate to the largest number that can be stored on the computer. 
    !*****************************************************************

    if (CorrectedRate%value .lt. 1e10) then
        CorrectedCountRate = CorrectedRate%value
        countRateError = (correctedRate%errorMinus + correctedRate%errorPlus) / 2.0d0
    else
        CorrectedCountRate = -1.0d0
    end if
   

!write(*, *) "Natural count rate=", rawCountRate, " corrected count rate=", CorrectedCountRate, "corrected error=", countRateError

end function CorrectedCountRate


end module ommag_mod












